/*
 *  SPDX-FileCopyrightText: 2020 Dmitry Kazakov <dimula73@gmail.com>
 *
 *  SPDX-License-Identifier: GPL-2.0-or-later
 */

#include "KoCanvasResourcesInterface.h"

KoCanvasResourcesInterface::~KoCanvasResourcesInterface()
{
}
